package main

import (
	"context"
	"github.com/disgoorg/disgo"
	"github.com/disgoorg/disgo/bot"
	"github.com/disgoorg/disgo/cache"
	"github.com/disgoorg/disgo/gateway"
	"github.com/disgoorg/log"
	"grow.rievo.dev/discordBots/cmd/funbot/command"
	"grow.rievo.dev/discordBots/cmd/funbot/config"
	"os"
	"os/signal"
	"strings"
	"syscall"
)

func main() {
	log.SetLevel(log.LevelInfo)
	log.Info("starting funbot...")
	log.Info("disgo version: ", disgo.Version)

	// permissions:
	// intent:
	client, err := disgo.New(config.Token,
		bot.WithGatewayConfigOpts(
			gateway.WithIntents(gateway.IntentsNone),
		),
		bot.WithCacheConfigOpts(
			cache.WithCaches(
				cache.FlagsNone,
			),
		),
		bot.WithEventListenerFunc(command.Listener),
	)
	if err != nil {
		log.Fatal("error while building disgo instance: ", err)
		return
	}

	defer client.Close(context.TODO())

	var globalComands bool
	if strings.ToUpper(config.GlobalCommands) == "TRUE" {
		globalComands = true
	}

	if globalComands {
		if _, err = client.Rest().SetGlobalCommands(client.ApplicationID(), config.Commands); err != nil {
			log.Fatal("error while registering commands: ", err)
		}
		if config.RegisterGuildID != 0 {
			if _, err = client.Rest().SetGuildCommands(client.ApplicationID(), config.RegisterGuildID, config.NoCommands); err != nil {
				log.Info("error deleting guild commands", err)
			}
		}
	} else {
		if _, err = client.Rest().SetGuildCommands(client.ApplicationID(), config.RegisterGuildID, config.Commands); err != nil {
			log.Fatal("error while registering commands: ", err)
		}
		if _, err = client.Rest().SetGlobalCommands(client.ApplicationID(), config.NoCommands); err != nil {
			log.Info("error deleting global commands", err)
		}
	}

	if err = client.OpenGateway(context.TODO()); err != nil {
		log.Fatal("error while connecting to gateway: ", err)
	}

	log.Infof("funbot is now running. Press CTRL-C to exit.")
	s := make(chan os.Signal, 1)
	signal.Notify(s, syscall.SIGINT, syscall.SIGTERM, os.Interrupt)
	<-s
}
